﻿using System;

using NATS.Client;

namespace TestCommon
{
    /// <summary>
    /// 연결 정보
    /// </summary>
    public class ConnectionInfo
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 최대 페이로드 - MaximumPayload

        /// <summary>
        /// 최대 페이로드
        /// </summary>
        public long MaximumPayload { get; private set; }

        #endregion
        #region 연결 상태 - State

        /// <summary>
        /// 연결 상태
        /// </summary>
        public ConnectionState State { get; private set; }

        #endregion
        #region 마지막 예외 - LastException

        /// <summary>
        /// 마지막 예외
        /// </summary>
        public Exception LastException { get; private set; }

        #endregion
        #region 발견된 서버 배열 - DiscoveredServerArray

        /// <summary>
        /// 발견된 서버 배열
        /// </summary>
        public string[] DiscoveredServerArray { get; private set; }

        #endregion
        #region 서버 배열 - ServerArray

        /// <summary>
        /// 서버 배열
        /// </summary>
        public string[] ServerArray { get; private set; }

        #endregion
        #region 연결 ID - ConnectedID

        /// <summary>
        /// 연결 ID
        /// </summary>
        public string ConnectedID { get; private set; }

        #endregion
        #region 연결 URL - ConnecedURL

        /// <summary>
        /// 연결 URL
        /// </summary>
        public string ConnecedURL { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - ConnectionInfo()

        /// <summary>
        /// 생성자
        /// </summary>
        private ConnectionInfo()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 연결 정보 구하기 - GetConnectionInfo(connection)

        /// <summary>
        /// 연결 정보 구하기
        /// </summary>
        /// <param name="connection">NATS 연결</param>
        /// <returns>연결 정보</returns>
        public static ConnectionInfo GetConnectionInfo(Connection connection)
        {
            ConnectionInfo info = new ConnectionInfo();

            info.MaximumPayload    = connection.MaxPayload;
            info.State             = (ConnectionState)(int)connection.State;
            info.LastException     = connection.LastError;
            info.DiscoveredServerArray = connection.DiscoveredServers;
            info.ServerArray           = connection.Servers;
            info.ConnectedID       = connection.ConnectedId;
            info.ConnecedURL       = connection.ConnectedUrl;

            return info;
        }

        #endregion
    }
}