﻿using System;

namespace TestCommon
{
    /// <summary>
    /// 메시지 수신시 이벤트 인자
    /// </summary>
    public class MessageReceivedEventArgs : EventArgs
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 서브젝트
        /// </summary>
        private string subject;

        /// <summary>
        /// 메시지 바이트 배열
        /// </summary>
        private byte[] messageByteArray;

        /// <summary>
        /// 응답 서브젝트
        /// </summary>
        private string replySubject;

        /// <summary>
        /// 응답 메시지 바이트 배열
        /// </summary>
        private byte[] replyMessageByteArray;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 서브젝트 - Subject

        /// <summary>
        /// 서브젝트
        /// </summary>
        public string Subject
        {
            get
            {
                return this.subject;
            }
        }

        #endregion
        #region 메시지 바이트 배열 - MessageByteArray

        /// <summary>
        /// 메시지 바이트 배열
        /// </summary>
        public byte[] MessageByteArray
        {
            get
            {
                return this.messageByteArray;
            }
        }

        #endregion
        #region 응답 서브젝트 - ReplySubject

        /// <summary>
        /// 응답 서브젝트
        /// </summary>
        public string ReplySubject
        {
            get
            {
                return this.replySubject;
            }
        }

        #endregion
        #region 응답 메시지 바이트 배열 - ReplyMessageByteArray

        /// <summary>
        /// 응답 메시지 바이트 배열
        /// </summary>
        public byte[] ReplyMessageByteArray
        {
            get
            {
                return this.replyMessageByteArray;
            }
            set
            {
                this.replyMessageByteArray = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MessageReceivedEventArgs(subject, messageByteArray, replySubject)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="subject">서브젝트</param>
        /// <param name="messageByteArray">메시지 바이트 배열</param>
        /// <param name="replySubject">응답 서브젝트</param>
        public MessageReceivedEventArgs(string subject, byte[] messageByteArray, string replySubject)
        {
            if(string.IsNullOrWhiteSpace(subject))
            {
                throw new ArgumentException("subject가 null이거나 공백 문자열 입니다.");
            }

            this.subject = subject;

            this.messageByteArray = messageByteArray;

            this.replySubject = string.IsNullOrWhiteSpace(replySubject) ? null : replySubject.Trim();

            this.replyMessageByteArray = null;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - MessageReceivedEventArgs()

        /// <summary>
        /// 생성자
        /// </summary>
        private MessageReceivedEventArgs()
        {
        }

        #endregion
    }
}