﻿using System;

using NATS.Client;

namespace TestCommon
{
    /// <summary>
    /// 구독 정보
    /// </summary>
    public class SubscriptionInfo : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 서브젝트
        /// </summary>
        private string subject;

        /// <summary>
        /// 큐
        /// </summary>
        private string queue;

        /// <summary>
        /// 구독
        /// </summary>
        private IAsyncSubscription subscription;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 서브젝트 - Subject

        /// <summary>
        /// 서브젝트
        /// </summary>
        public string Subject
        {
            get
            {
                return this.subject;
            }
        }

        #endregion
        #region 큐 - Queue

        /// <summary>
        /// 큐
        /// </summary>
        public string Queue
        {
            get
            {
                return this.queue;
            }
        }

        #endregion
        #region 구독 - Subscription

        /// <summary>
        /// 구독
        /// </summary>
        public IAsyncSubscription Subscription
        {
            get
            {
                return this.subscription;
            }
        }

        #endregion

        #region 키 - Key

        /// <summary>
        /// 키
        /// </summary>
        public string Key
        {
            get
            {
                if(this.queue == null)
                {
                    return this.subject;
                }
                else
                {
                    return GetKey(this.subject, this.queue);
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SubscriptionInfo(subject, queue)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="subject">서브젝트</param>
        /// <param name="queue">큐</param>
        public SubscriptionInfo(string subject, string queue)
        {
            if(string.IsNullOrWhiteSpace(subject))
            {
                throw new ArgumentException("subject가 null이거나 공백 문자열 입니다.");
            }

            this.subject = subject;

            this.queue = string.IsNullOrWhiteSpace(queue) ? null : queue.Trim();

            this.subscription = null;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - SubscriptionInfo()

        /// <summary>
        /// 생성자
        /// </summary>
        private SubscriptionInfo()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 키 구하기 - GetKey(subject, queue)

        /// <summary>
        /// 키 구하기
        /// </summary>
        /// <param name="subject">서브젝트</param>
        /// <param name="queue">큐</param>
        /// <returns>키</returns>
        public static string GetKey(string subject, string queue)
        {
            return $"{subject}_{queue}";
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 구독 시작하기 - StartSubscription(connection, receivedEventHandler)

        /// <summary>
        /// 구독 시작하기
        /// </summary>
        /// <param name="connection">NATS 연결</param>
        /// <param name="receivedEventHandler">RECEIVED 이벤트 핸들러</param>
        public void StartSubscription(IConnection connection, EventHandler<MsgHandlerEventArgs> receivedEventHandler)
        {
            this.subscription = connection.SubscribeAsync(this.subject, this.queue, receivedEventHandler);
        }

        #endregion
        #region 구독 중단하기 - StopSubscription()

        /// <summary>
        /// 구독 중단하기
        /// </summary>
        public void StopSubscription()
        {
            if(this.subscription != null)
            {
                this.subscription.Unsubscribe();

                this.subscription.Dispose();

                this.subscription = null;
            }
        }

        #endregion

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            StopSubscription();
        }

        #endregion
    }
}